package com.jpsoft.shinestar.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.shinestar.modules.common.utils.PojoUtils;
import com.jpsoft.shinestar.modules.common.dto.Sort;
import com.jpsoft.shinestar.modules.common.dto.MessageResult;
import com.jpsoft.shinestar.modules.base.entity.ActivityInfo;
import com.jpsoft.shinestar.modules.base.service.ActivityInfoService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/base/activityInfo")
@Api(description = "activityInfo")
public class ActivityInfoController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private ActivityInfoService activityInfoService;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<ActivityInfo> create(){
        MessageResult<ActivityInfo> msgResult = new MessageResult<>();

        ActivityInfo activityInfo = new ActivityInfo();

        msgResult.setData(activityInfo);
        msgResult.setResult(true);

        return msgResult;
    }
    
    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<ActivityInfo> add(@RequestBody ActivityInfo activityInfo,@RequestAttribute String subject){
        MessageResult<ActivityInfo> msgResult = new MessageResult<>();

        try {
            activityInfo.setId(UUID.randomUUID().toString());
            activityInfo.setDelFlag(false);
            activityInfo.setCreateBy(subject);
            activityInfo.setCreateTime(new Date());
            
            int affectCount = activityInfoService.insert(activityInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(activityInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<ActivityInfo> edit(@PathVariable("id") String id){
        MessageResult<ActivityInfo> msgResult = new MessageResult<>();

        try {
            ActivityInfo activityInfo = activityInfoService.get(id);

            if (activityInfo != null) {
                msgResult.setResult(true);
                msgResult.setData(activityInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<ActivityInfo> update(@RequestBody ActivityInfo activityInfo,@RequestAttribute String subject){
        MessageResult<ActivityInfo> msgResult = new MessageResult<>();

        try {
            activityInfo.setUpdateBy(subject);
            activityInfo.setUpdateTime(new Date());
            
            int affectCount = activityInfoService.update(activityInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(activityInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            ActivityInfo activityInfo = activityInfoService.get(id);
            activityInfo.setDelFlag(true);
            activityInfo.setUpdateBy(subject);
            activityInfo.setUpdateTime(new Date());

            int affectCount = activityInfoService.update(activityInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                ActivityInfo activityInfo = activityInfoService.get(id);
                activityInfo.setDelFlag(true);
                activityInfo.setUpdateBy(subject);
                activityInfo.setUpdateTime(new Date());

                affectCount += activityInfoService.update(activityInfo);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String title, String applicant,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("create_time","desc"));

        if (StringUtils.isNotEmpty(title)) {
            searchParams.put("title","%" + title + "%");
        }
        if (StringUtils.isNotEmpty(applicant)) {
            searchParams.put("applicant","%" + applicant + "%");
        }

        Page<ActivityInfo> page = activityInfoService.pageSearch(searchParams,pageIndex,pageSize,true,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }
}
