package com.jpsoft.shinestar.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.shinestar.modules.base.entity.PersonInfo;
import com.jpsoft.shinestar.modules.base.service.PersonInfoService;
import com.jpsoft.shinestar.modules.common.dto.Sort;
import com.jpsoft.shinestar.modules.common.dto.MessageResult;
import com.jpsoft.shinestar.modules.base.entity.EpidemicReport;
import com.jpsoft.shinestar.modules.base.service.EpidemicReportService;
import com.jpsoft.shinestar.modules.common.utils.PojoUtils;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.apache.ibatis.mapping.ResultMap;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/epidemicReport")
public class EpidemicReportController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private EpidemicReportService epidemicReportService;

    @Autowired
    private PersonInfoService personInfoService;

    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<EpidemicReport> add(@ModelAttribute EpidemicReport epidemicReport){
        MessageResult<EpidemicReport> msgResult = new MessageResult<>();

        try {
            epidemicReport.setId(UUID.randomUUID().toString());

            int affectCount = epidemicReportService.insert(epidemicReport);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(epidemicReport);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("detail/{id}")
    public MessageResult<EpidemicReport> detail(@PathVariable("id") String id){
        MessageResult<EpidemicReport> msgResult = new MessageResult<>();

        try {
            EpidemicReport epidemicReport = epidemicReportService.get(id);

            if (epidemicReport != null) {
                msgResult.setResult(true);
                msgResult.setData(epidemicReport);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<EpidemicReport> update(@ModelAttribute EpidemicReport epidemicReport){
        MessageResult<EpidemicReport> msgResult = new MessageResult<>();

        try {
            int affectCount = epidemicReportService.update(epidemicReport);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(epidemicReport);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="删除用户")
    @PostMapping("delete/{id}")
    public MessageResult<EpidemicReport> delete(@PathVariable("id") String id){
        MessageResult<EpidemicReport> msgResult = new MessageResult<>();

        try {
            int affectCount = epidemicReportService.delete(id);

            if (affectCount > 0) {
                msgResult.setResult(true);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete/{id}")
    public MessageResult<EpidemicReport> batchDelete(@PathVariable("id") String id){
        MessageResult<EpidemicReport> msgResult = new MessageResult<>();

        msgResult.setResult(false);
        msgResult.setMessage("未实现");

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "list",method = RequestMethod.POST)
    public MessageResult<Map> list(
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            HttpServletRequest request){
        String subject = (String)request.getAttribute("subject");

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("create_time","desc"));

        Page<EpidemicReport> page = epidemicReportService.pageSearch(searchParams,pageIndex,pageSize,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }
}
