package com.jpsoft.shinestar.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.shinestar.modules.common.utils.PojoUtils;
import com.jpsoft.shinestar.modules.common.dto.Sort;
import com.jpsoft.shinestar.modules.common.dto.MessageResult;
import com.jpsoft.shinestar.modules.base.entity.MessageNoticeCompany;
import com.jpsoft.shinestar.modules.base.service.MessageNoticeCompanyService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;

@RestController
@RequestMapping("/messageNoticeCompany")
@Api(description = "messageNoticeCompany")
public class MessageNoticeCompanyController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private MessageNoticeCompanyService messageNoticeCompanyService;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<MessageNoticeCompany> create(){
        MessageResult<MessageNoticeCompany> msgResult = new MessageResult<>();

        MessageNoticeCompany messageNoticeCompany = new MessageNoticeCompany();

        msgResult.setData(messageNoticeCompany);
        msgResult.setResult(true);

        return msgResult;
    }
    
    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<MessageNoticeCompany> add(@RequestBody MessageNoticeCompany messageNoticeCompany,@RequestAttribute String subject){
        MessageResult<MessageNoticeCompany> msgResult = new MessageResult<>();

        try {
            messageNoticeCompany.setId(UUID.randomUUID().toString());
            messageNoticeCompany.setDelFlag(false);
            messageNoticeCompany.setCreateBy(subject);
            messageNoticeCompany.setCreateTime(new Date());
            
            int affectCount = messageNoticeCompanyService.insert(messageNoticeCompany);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(messageNoticeCompany);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<MessageNoticeCompany> edit(@PathVariable("id") String id){
        MessageResult<MessageNoticeCompany> msgResult = new MessageResult<>();

        try {
            MessageNoticeCompany messageNoticeCompany = messageNoticeCompanyService.get(id);

            if (messageNoticeCompany != null) {
                msgResult.setResult(true);
                msgResult.setData(messageNoticeCompany);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<MessageNoticeCompany> update(@RequestBody MessageNoticeCompany messageNoticeCompany,@RequestAttribute String subject){
        MessageResult<MessageNoticeCompany> msgResult = new MessageResult<>();

        try {
            messageNoticeCompany.setUpdateBy(subject);
            messageNoticeCompany.setUpdateTime(new Date());
            
            int affectCount = messageNoticeCompanyService.update(messageNoticeCompany);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(messageNoticeCompany);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            MessageNoticeCompany messageNoticeCompany = messageNoticeCompanyService.get(id);
            messageNoticeCompany.setDelFlag(true);
            messageNoticeCompany.setUpdateBy(subject);
            messageNoticeCompany.setUpdateTime(new Date());

            int affectCount = messageNoticeCompanyService.update(messageNoticeCompany);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                MessageNoticeCompany messageNoticeCompany = messageNoticeCompanyService.get(id);
                messageNoticeCompany.setDelFlag(true);
                messageNoticeCompany.setUpdateBy(subject);
                messageNoticeCompany.setUpdateTime(new Date());

                affectCount += messageNoticeCompanyService.update(messageNoticeCompany);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String id,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("id_","asc"));

        if (StringUtils.isNotEmpty(id)) {
            searchParams.put("id","%" + id + "%");
        }

        Page<MessageNoticeCompany> page = messageNoticeCompanyService.pageSearch(searchParams,pageIndex,pageSize,true,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }
}
