package com.jpsoft.shinestar.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.shinestar.modules.common.utils.PojoUtils;
import com.jpsoft.shinestar.modules.common.dto.Sort;
import com.jpsoft.shinestar.modules.common.dto.MessageResult;
import com.jpsoft.shinestar.modules.base.entity.MessageReport;
import com.jpsoft.shinestar.modules.base.service.MessageReportService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;

@RestController
@RequestMapping("/messageReport")
@Api(description = "messageReport")
public class MessageReportController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private MessageReportService messageReportService;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<MessageReport> create(){
        MessageResult<MessageReport> msgResult = new MessageResult<>();

        MessageReport messageReport = new MessageReport();

        msgResult.setData(messageReport);
        msgResult.setResult(true);

        return msgResult;
    }
    
    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<MessageReport> add(@RequestBody MessageReport messageReport,@RequestAttribute String subject){
        MessageResult<MessageReport> msgResult = new MessageResult<>();

        try {
            messageReport.setId(UUID.randomUUID().toString());
            messageReport.setDelFlag(false);
            messageReport.setCreateBy(subject);
            messageReport.setCreateTime(new Date());
            
            int affectCount = messageReportService.insert(messageReport);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(messageReport);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<MessageReport> edit(@PathVariable("id") String id){
        MessageResult<MessageReport> msgResult = new MessageResult<>();

        try {
            MessageReport messageReport = messageReportService.get(id);

            if (messageReport != null) {
                msgResult.setResult(true);
                msgResult.setData(messageReport);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<MessageReport> update(@RequestBody MessageReport messageReport,@RequestAttribute String subject){
        MessageResult<MessageReport> msgResult = new MessageResult<>();

        try {
            messageReport.setUpdateBy(subject);
            messageReport.setUpdateTime(new Date());
            
            int affectCount = messageReportService.update(messageReport);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(messageReport);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            MessageReport messageReport = messageReportService.get(id);
            messageReport.setDelFlag(true);
            messageReport.setUpdateBy(subject);
            messageReport.setUpdateTime(new Date());

            int affectCount = messageReportService.update(messageReport);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                MessageReport messageReport = messageReportService.get(id);
                messageReport.setDelFlag(true);
                messageReport.setUpdateBy(subject);
                messageReport.setUpdateTime(new Date());

                affectCount += messageReportService.update(messageReport);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String title,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("create_time","desc"));

        if (StringUtils.isNotEmpty(title)) {
            searchParams.put("title","%" + title + "%");
        }

        Page<MessageReport> page = messageReportService.pageSearch(searchParams,pageIndex,pageSize,true,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }
}
