package com.jpsoft.shinestar.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.shinestar.modules.common.utils.PojoUtils;
import com.jpsoft.shinestar.modules.common.dto.Sort;
import com.jpsoft.shinestar.modules.common.dto.MessageResult;
import com.jpsoft.shinestar.modules.base.entity.MessageSendCondition;
import com.jpsoft.shinestar.modules.base.service.MessageSendConditionService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/base/messageSendCondition")
@Api(description = "消息发送条件设置7")
public class MessageSendConditionController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private MessageSendConditionService messageSendConditionService;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<MessageSendCondition> create(){
        MessageResult<MessageSendCondition> msgResult = new MessageResult<>();

        MessageSendCondition messageSendCondition = new MessageSendCondition();

        msgResult.setData(messageSendCondition);
        msgResult.setResult(true);

        return msgResult;
    }
    
    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<MessageSendCondition> add(@RequestBody MessageSendCondition messageSendCondition,@RequestAttribute String subject){
        MessageResult<MessageSendCondition> msgResult = new MessageResult<>();

        try {
            messageSendCondition.setId(UUID.randomUUID().toString());
            messageSendCondition.setDelFlag(false);
            messageSendCondition.setCreateBy(subject);
            messageSendCondition.setCreateTime(new Date());
            
            int affectCount = messageSendConditionService.insert(messageSendCondition);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(messageSendCondition);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<MessageSendCondition> edit(@PathVariable("id") String id){
        MessageResult<MessageSendCondition> msgResult = new MessageResult<>();

        try {
            MessageSendCondition messageSendCondition = messageSendConditionService.get(id);

            if (messageSendCondition != null) {
                msgResult.setResult(true);
                msgResult.setData(messageSendCondition);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<MessageSendCondition> update(@RequestBody MessageSendCondition messageSendCondition,@RequestAttribute String subject){
        MessageResult<MessageSendCondition> msgResult = new MessageResult<>();

        try {
            messageSendCondition.setUpdateBy(subject);
            messageSendCondition.setUpdateTime(new Date());
            
            int affectCount = messageSendConditionService.update(messageSendCondition);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(messageSendCondition);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            MessageSendCondition messageSendCondition = messageSendConditionService.get(id);
            messageSendCondition.setDelFlag(true);
            messageSendCondition.setUpdateBy(subject);
            messageSendCondition.setUpdateTime(new Date());

            int affectCount = messageSendConditionService.update(messageSendCondition);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                MessageSendCondition messageSendCondition = messageSendConditionService.get(id);
                messageSendCondition.setDelFlag(true);
                messageSendCondition.setUpdateBy(subject);
                messageSendCondition.setUpdateTime(new Date());

                affectCount += messageSendConditionService.update(messageSendCondition);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String messageId,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("b.create_time","asc"));

        if (StringUtils.isNotEmpty(messageId)) {
            searchParams.put("messageId", messageId);
        }

        Page<MessageSendCondition> page = messageSendConditionService.pageSearch(searchParams,pageIndex,pageSize,true,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }
}
