package com.jpsoft.shinestar.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.shinestar.modules.common.dto.Sort;
import com.jpsoft.shinestar.modules.base.entity.ProblemFeedback;
import com.jpsoft.shinestar.modules.base.service.ProblemFeedbackService;
import com.jpsoft.shinestar.modules.common.dto.MessageResult;
import com.jpsoft.shinestar.modules.common.utils.PojoUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.*;
/**
 * 问题反馈
 * sz
 * **/
@RestController
@RequestMapping("/base/problemFeedback")
@Api(description = "problemFeedback")
public class ProblemFeedbackController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private ProblemFeedbackService problemFeedbackService;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<ProblemFeedback> create(){
        MessageResult<ProblemFeedback> msgResult = new MessageResult<>();

        ProblemFeedback problemFeedback = new ProblemFeedback();

        msgResult.setData(problemFeedback);
        msgResult.setResult(true);

        return msgResult;
    }

    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<ProblemFeedback> add(@RequestBody ProblemFeedback problemFeedback,@RequestAttribute String subject){
        MessageResult<ProblemFeedback> msgResult = new MessageResult<>();

        try {
            problemFeedback.setId(UUID.randomUUID().toString());
            problemFeedback.setDelFlag(false);
            problemFeedback.setCreateBy(subject);
            problemFeedback.setCreateTime(new Date());

            int affectCount = problemFeedbackService.insert(problemFeedback);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(problemFeedback);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<ProblemFeedback> edit(@PathVariable("id") String id){
        MessageResult<ProblemFeedback> msgResult = new MessageResult<>();

        try {
            ProblemFeedback problemFeedback = problemFeedbackService.get(id);

            if (problemFeedback != null) {
                msgResult.setResult(true);
                msgResult.setData(problemFeedback);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<ProblemFeedback> update(@RequestBody ProblemFeedback problemFeedback,@RequestAttribute String subject){
        MessageResult<ProblemFeedback> msgResult = new MessageResult<>();

        try {
            problemFeedback.setUpdateBy(subject);
            problemFeedback.setUpdateTime(new Date());

            int affectCount = problemFeedbackService.update(problemFeedback);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(problemFeedback);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="删除")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            ProblemFeedback problemFeedback = problemFeedbackService.get(id);
            problemFeedback.setDelFlag(true);
            problemFeedback.setUpdateBy(subject);
            problemFeedback.setUpdateTime(new Date());

            int affectCount = problemFeedbackService.update(problemFeedback);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                ProblemFeedback problemFeedback = problemFeedbackService.get(id);
                problemFeedback.setDelFlag(true);
                problemFeedback.setUpdateBy(subject);
                problemFeedback.setUpdateTime(new Date());

                affectCount += problemFeedbackService.update(problemFeedback);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String id,
            @RequestParam(value="name",defaultValue="") String name,
            @RequestParam(value="phone",defaultValue="") String phone,
            @RequestParam(value="createTimeRanges",defaultValue="") String createTimeRanges,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("a.create_time","desc"));

        if (StringUtils.isNotEmpty(id)) {
            searchParams.put("id","%" + id + "%");
        }

        if (StringUtils.isNotEmpty(name)) {
            searchParams.put("name", "%" + name + "%");
        }

        if (StringUtils.isNotEmpty(phone)) {
            searchParams.put("phone", "%" + phone + "%");
        }

        if (StringUtils.isNotEmpty(createTimeRanges)) {
            String[] timeRangeArray = createTimeRanges.split(",");
            String beginTime = "";
            String endTime = "";
            if (timeRangeArray.length == 1) {
                beginTime = timeRangeArray[0];
                beginTime += " 00:00:00";
            } else if (timeRangeArray.length == 2) {
                beginTime = timeRangeArray[0];
                endTime = timeRangeArray[1];
                beginTime += " 00:00:00";
                endTime += " 23:59:59";
            }

            searchParams.put("beginTime", beginTime);
            searchParams.put("endTime", endTime);
        }

        Page<ProblemFeedback> page = problemFeedbackService.pageSearch(searchParams,pageIndex,pageSize,true,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }
}
