package com.jpsoft.shinestar.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.shinestar.modules.common.utils.PojoUtils;
import com.jpsoft.shinestar.modules.common.dto.Sort;
import com.jpsoft.shinestar.modules.common.dto.MessageResult;
import com.jpsoft.shinestar.modules.base.entity.SceneInfo;
import com.jpsoft.shinestar.modules.base.service.SceneInfoService;
import com.jpsoft.shinestar.modules.common.utils.PojoUtils;
import com.jpsoft.shinestar.modules.sys.entity.User;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * 场景值
 * sz
 * **/
@RestController
@RequestMapping("/base/sceneInfo")
@Api(description = "sceneInfo")
public class SceneInfoController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private SceneInfoService sceneInfoService;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<SceneInfo> create(){
        MessageResult<SceneInfo> msgResult = new MessageResult<>();

        SceneInfo sceneInfo = new SceneInfo();

        msgResult.setData(sceneInfo);
        msgResult.setResult(true);

        return msgResult;
    }
    
    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<SceneInfo> add(@RequestBody SceneInfo sceneInfo,@RequestAttribute String subject){
        MessageResult<SceneInfo> msgResult = new MessageResult<>();

        try {
            sceneInfo.setId(UUID.randomUUID().toString());
            sceneInfo.setDelFlag(false);
            sceneInfo.setCreateBy(subject);
            sceneInfo.setCreateTime(new Date());
            
            int affectCount = sceneInfoService.insert(sceneInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(sceneInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<SceneInfo> edit(@PathVariable("id") String id){
        MessageResult<SceneInfo> msgResult = new MessageResult<>();

        try {
            SceneInfo sceneInfo = sceneInfoService.get(id);

            if (sceneInfo != null) {
                msgResult.setResult(true);
                msgResult.setData(sceneInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<SceneInfo> update(@RequestBody SceneInfo sceneInfo,@RequestAttribute String subject){
        MessageResult<SceneInfo> msgResult = new MessageResult<>();

        try {
            sceneInfo.setUpdateBy(subject);
            sceneInfo.setUpdateTime(new Date());
            
            int affectCount = sceneInfoService.update(sceneInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(sceneInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            SceneInfo sceneInfo = sceneInfoService.get(id);
            sceneInfo.setDelFlag(true);
            sceneInfo.setUpdateBy(subject);
            sceneInfo.setUpdateTime(new Date());

            int affectCount = sceneInfoService.update(sceneInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                SceneInfo sceneInfo = sceneInfoService.get(id);
                sceneInfo.setDelFlag(true);
                sceneInfo.setUpdateBy(subject);
                sceneInfo.setUpdateTime(new Date());

                affectCount += sceneInfoService.update(sceneInfo);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String id,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("id_","asc"));

        if (StringUtils.isNotEmpty(id)) {
            searchParams.put("id","%" + id + "%");
        }

        Page<SceneInfo> page = sceneInfoService.pageSearch(searchParams,pageIndex,pageSize,true,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }

    @ApiOperation(value="场景值列表")
    @RequestMapping(value = "list",method = RequestMethod.POST)
    public MessageResult<List<SceneInfo>> list(String type,@RequestAttribute String subject){
        MessageResult<List<SceneInfo>> msgResult = new MessageResult<>();
        List<SceneInfo> list = sceneInfoService.list();

        msgResult.setResult(true);
        msgResult.setData(list);
        return msgResult;
    }
}
