package com.jpsoft.shinestar.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.shinestar.modules.common.dto.Sort;
import com.jpsoft.shinestar.modules.common.dto.MessageResult;
import com.jpsoft.shinestar.modules.base.entity.WorkShiftCompany;
import com.jpsoft.shinestar.modules.base.service.WorkShiftCompanyService;
import com.jpsoft.shinestar.modules.common.utils.PojoUtils;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.apache.ibatis.mapping.ResultMap;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/base/workShiftCompany")
public class WorkShiftCompanyController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private WorkShiftCompanyService workShiftCompanyService;

    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<WorkShiftCompany> add(@ModelAttribute WorkShiftCompany workShiftCompany){
        MessageResult<WorkShiftCompany> msgResult = new MessageResult<>();

        try {
            workShiftCompany.setId(UUID.randomUUID().toString());

            int affectCount = workShiftCompanyService.insert(workShiftCompany);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(workShiftCompany);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("detail/{id}")
    public MessageResult<WorkShiftCompany> detail(@PathVariable("id") String id){
        MessageResult<WorkShiftCompany> msgResult = new MessageResult<>();

        try {
            WorkShiftCompany workShiftCompany = workShiftCompanyService.get(id);

            if (workShiftCompany != null) {
                msgResult.setResult(true);
                msgResult.setData(workShiftCompany);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<WorkShiftCompany> update(@ModelAttribute WorkShiftCompany workShiftCompany){
        MessageResult<WorkShiftCompany> msgResult = new MessageResult<>();

        try {
            int affectCount = workShiftCompanyService.update(workShiftCompany);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(workShiftCompany);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="删除用户")
    @PostMapping("delete/{id}")
    public MessageResult<WorkShiftCompany> delete(@PathVariable("id") String id){
        MessageResult<WorkShiftCompany> msgResult = new MessageResult<>();

        try {
            int affectCount = workShiftCompanyService.delete(id);

            if (affectCount > 0) {
                msgResult.setResult(true);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete/{id}")
    public MessageResult<WorkShiftCompany> batchDelete(@PathVariable("id") String id){
        MessageResult<WorkShiftCompany> msgResult = new MessageResult<>();

        msgResult.setResult(false);
        msgResult.setMessage("未实现");

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String id,
            @RequestParam(name="pageIndex",defaultValue = "1") int pageIndex,
            @RequestParam(name="pageSize",defaultValue = "10") int pageSize,
            HttpServletRequest request){
        String subject = (String)request.getAttribute("subject");

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParms = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("create_time","desc"));

        if (StringUtils.isNotEmpty(id)) {
            searchParms.put("id","%" + id + "%");
        }

        Page<WorkShiftCompany> page = workShiftCompanyService.pageSearch(searchParms,pageIndex,pageSize, true,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }
}
