package com.jpsoft.shinestar.modules.business.controller;

import com.github.pagehelper.Page;
import com.jpsoft.shinestar.modules.common.dto.Sort;
import com.jpsoft.shinestar.modules.common.dto.MessageResult;
import com.jpsoft.shinestar.modules.business.entity.MedicalRecords;
import com.jpsoft.shinestar.modules.business.service.MedicalRecordsService;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.apache.ibatis.mapping.ResultMap;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/medicalRecords")
public class MedicalRecordsController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private MedicalRecordsService medicalRecordsService;

    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<MedicalRecords> add(@ModelAttribute MedicalRecords medicalRecords){
        MessageResult<MedicalRecords> msgResult = new MessageResult<>();

        try {
            medicalRecords.setId(UUID.randomUUID().toString());

            int affectCount = medicalRecordsService.insert(medicalRecords);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(medicalRecords);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("detail/{id}")
    public MessageResult<MedicalRecords> detail(@PathVariable("id") String id){
        MessageResult<MedicalRecords> msgResult = new MessageResult<>();

        try {
            MedicalRecords medicalRecords = medicalRecordsService.get(id);

            if (medicalRecords != null) {
                msgResult.setResult(true);
                msgResult.setData(medicalRecords);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<MedicalRecords> update(@ModelAttribute MedicalRecords medicalRecords){
        MessageResult<MedicalRecords> msgResult = new MessageResult<>();

        try {
            int affectCount = medicalRecordsService.update(medicalRecords);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(medicalRecords);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="删除用户")
    @PostMapping("delete/{id}")
    public MessageResult<MedicalRecords> delete(@PathVariable("id") String id){
        MessageResult<MedicalRecords> msgResult = new MessageResult<>();

        try {
            int affectCount = medicalRecordsService.delete(id);

            if (affectCount > 0) {
                msgResult.setResult(true);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete/{id}")
    public MessageResult<MedicalRecords> batchDelete(@PathVariable("id") String id){
        MessageResult<MedicalRecords> msgResult = new MessageResult<>();

        msgResult.setResult(false);
        msgResult.setMessage("未实现");

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "list",method = RequestMethod.POST)
    public MessageResult<Map> list(
            String id,
            @RequestParam(value="draw",defaultValue="1") int draw,
            @RequestParam(value="start",defaultValue="1") int start,
            @RequestParam(value="length",defaultValue="20") int length,
            HttpServletRequest request){
        String subject = (String)request.getAttribute("subject");

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParms = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("id_","asc"));

        if (StringUtils.isNotEmpty(id)) {
            searchParms.put("id","%" + id + "%");
        }

        int pageNum = start / length;

        if (start % length != 0) {
            pageNum++;
        }

        int pageSize = length;

        Page<MedicalRecords> page = medicalRecordsService.pageSearch(searchParms,pageNum,pageSize,sortList);

        HashMap<String, Object> wrapper = new HashMap<String, Object>();
        wrapper.put("draw",draw);
        wrapper.put("recordsTotal", page.getTotal());
        wrapper.put("recordsFiltered", page.getTotal());
        wrapper.put("data", page.getResult());

        msgResult.setResult(true);
        msgResult.setData(wrapper);

        return msgResult;
    }
}
